library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

-- counter M=27.
-- Sampling rate: 115200 bps x 16 oversamplings = 1843200 ticks/s.
-- 50MHz / 1843200 ticks = 27.1267361111 counts.
-- This means that 27 clock system (from 0 to 26)is equivalent to 1 clock tick.

entity mod_27 is
	generic(
		N: integer := 5;	-- number of bits to count from 0 to 26.
		M: integer := 27 	-- count from (0 to 26).
	);
	port(
		clk, reset: in std_logic;
		max_tick: out std_logic;
		q: out std_logic_vector(N-1 downto 0)
	);
end entity mod_27;

architecture arch_mod_27 of mod_27 is
	signal r_reg: unsigned(N-1 downto 0);
	signal r_next: unsigned(N-1 downto 0);
begin
	sync_proc: process(clk, reset)
	begin
		if (reset = '1') then
			r_reg <= (others => '0');
		elsif (rising_edge(clk)) then
			r_reg <= r_next;
		end if;
	end process sync_proc;

	-- next state logic
	r_next <= (others => '0') when r_reg=(M-1) else r_reg + 1;

	-- output logic
	q <= std_logic_vector(r_reg);
	max_tick <= '1' when r_reg=(M-1) else '0';
end architecture arch_mod_27;
